var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { request } from './logger';
const getSystemInfo = () => {
    return new Promise((resolve) => {
        my.getSystemInfo({
            success: (systemInfo) => {
                resolve(systemInfo);
            },
            fail: () => { },
        });
    });
};
const createSlackLogMessage = (configs = null, appName, eId = '') => __awaiter(void 0, void 0, void 0, function* () {
    if (!configs)
        return '';
    const d = new Date();
    const time = d.toLocaleTimeString();
    const date = d.toLocaleDateString();
    const system = yield getSystemInfo();
    const { app, platform } = system;
    return [
        '```',
        `[${appName}] - ${eId}`,
        `Platform: ${app} - ${platform}`,
        `${date} ${time}`,
        '\nLog details: ',
        JSON.stringify(configs, null, 2),
        '```',
    ]
        .filter(Boolean)
        .join('\n');
});
export const sendMessageLogToSlack = (msg, appName, slackUrl) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        if (!msg || !slackUrl)
            return;
        msg = yield createSlackLogMessage(msg, appName);
        const blocks = [
            {
                type: 'section',
                text: {
                    type: 'mrkdwn',
                    text: msg,
                },
            },
        ];
        yield request({
            url: slackUrl,
            method: 'POST',
            headers: {
                'Content-type': 'application/json',
            },
            data: {
                blocks,
            },
        });
    }
    catch (error) {
        console.log(error);
    }
});
